import * as anchor from "@project-serum/anchor";
import { PublicKey, TransactionInstruction, Signer, Keypair } from "@solana/web3.js";
import BN from "bn.js";
import { Program, Provider } from "@project-serum/anchor";
import { SplTokenBondingIDL, CurveV0, ProgramStateV0 } from "./generated/spl-token-bonding";
import { AccountInfo, MintInfo } from "@solana/spl-token";
import { Curve } from "./curves";
import { InstructionResult } from "@strata-foundation/spl-utils";
export * from "./generated/spl-token-bonding";
export * from "./curves";
interface CurveConfig {
    toRawConfig(): CurveV0;
}
interface PrimitiveCurve {
    toRawPrimitiveConfig(): any;
}
export declare function toU128(num: number | BN): BN;
export declare class ExponentialCurveConfig implements CurveConfig, PrimitiveCurve {
    c: BN;
    b: BN;
    pow: number;
    frac: number;
    constructor({ c, b, pow, frac }: {
        c?: number | BN;
        b?: number | BN;
        pow?: number;
        frac?: number;
    });
    toRawPrimitiveConfig(): any;
    toRawConfig(): CurveV0;
}
export declare class TimeCurveConfig implements CurveConfig {
    curves: {
        curve: PrimitiveCurve;
        offset: BN;
    }[];
    addCurve(timeOffset: number, curve: PrimitiveCurve): TimeCurveConfig;
    toRawConfig(): CurveV0;
}
export interface InitializeCurveArgs {
    config: CurveConfig;
    payer?: PublicKey;
}
export interface CreateTokenBondingArgs {
    payer?: PublicKey;
    curve: PublicKey;
    baseMint: PublicKey;
    targetMint?: PublicKey;
    targetMintDecimals?: number;
    buyBaseRoyalties?: PublicKey;
    buyBaseRoyaltiesOwner?: PublicKey;
    buyTargetRoyalties?: PublicKey;
    buyTargetRoyaltiesOwner?: PublicKey;
    sellBaseRoyalties?: PublicKey;
    sellBaseRoyaltiesOwner?: PublicKey;
    sellTargetRoyalties?: PublicKey;
    sellTargetRoyaltiesOwner?: PublicKey;
    authority?: PublicKey;
    baseStorage?: PublicKey;
    buyBaseRoyaltyPercentage: number;
    buyTargetRoyaltyPercentage: number;
    sellBaseRoyaltyPercentage: number;
    sellTargetRoyaltyPercentage: number;
    mintCap?: BN;
    purchaseCap?: BN;
    goLiveDate?: Date;
    freezeBuyDate?: Date;
    buyFrozen?: boolean;
    index?: number;
}
export interface UpdateTokenBondingArgs {
    tokenBonding: PublicKey;
    buyBaseRoyaltyPercentage?: number;
    buyTargetRoyaltyPercentage?: number;
    sellBaseRoyaltyPercentage?: number;
    sellTargetRoyaltyPercentage?: number;
    buyBaseRoyalties?: PublicKey;
    buyTargetRoyalties?: PublicKey;
    sellBaseRoyalties?: PublicKey;
    sellTargetRoyalties?: PublicKey;
    authority?: PublicKey | null;
    buyFrozen?: boolean;
}
export interface BuyArgs {
    tokenBonding: PublicKey;
    payer?: PublicKey;
    source?: PublicKey;
    destination?: PublicKey;
    sourceAuthority?: PublicKey;
    desiredTargetAmount?: BN | number;
    baseAmount?: BN | number;
    slippage: number;
}
export interface SellArgs {
    tokenBonding: PublicKey;
    payer?: PublicKey;
    source?: PublicKey;
    destination?: PublicKey;
    sourceAuthority?: PublicKey;
    targetAmount: BN | number;
    slippage: number;
}
export declare class SplTokenBonding {
    program: Program<SplTokenBondingIDL>;
    provider: Provider;
    state: ProgramStateV0 | undefined;
    static ID: anchor.web3.PublicKey;
    static init(provider: Provider, splTokenBondingProgramId?: PublicKey): Promise<SplTokenBonding>;
    constructor(provider: Provider, program: Program<SplTokenBondingIDL>);
    get programId(): anchor.web3.PublicKey;
    get rpc(): anchor.RpcNamespace<SplTokenBondingIDL, {
        name: "initializeSolStorageV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "state";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeSolStorageV0Args";
            };
        }];
    } | {
        name: "buyWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyWrappedSolV0Args";
            };
        }];
    } | {
        name: "sellWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "owner";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellWrappedSolV0Args";
            };
        }];
    } | {
        name: "createCurveV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: true;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "CreateCurveV0Args";
            };
        }];
    } | {
        name: "initializeTokenBondingV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeTokenBondingV0Args";
            };
        }];
    } | {
        name: "closeTokenBondingV0";
        accounts: [{
            name: "refund";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [];
    } | {
        name: "updateTokenBondingV0";
        accounts: [{
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "UpdateTokenBondingV0Args";
            };
        }];
    } | {
        name: "buyV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyV0Args";
            };
        }];
    } | {
        name: "sellV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellV0Args";
            };
        }];
    }>;
    get instruction(): anchor.InstructionNamespace<SplTokenBondingIDL, {
        name: "initializeSolStorageV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "state";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeSolStorageV0Args";
            };
        }];
    } | {
        name: "buyWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyWrappedSolV0Args";
            };
        }];
    } | {
        name: "sellWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "owner";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellWrappedSolV0Args";
            };
        }];
    } | {
        name: "createCurveV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: true;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "CreateCurveV0Args";
            };
        }];
    } | {
        name: "initializeTokenBondingV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeTokenBondingV0Args";
            };
        }];
    } | {
        name: "closeTokenBondingV0";
        accounts: [{
            name: "refund";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [];
    } | {
        name: "updateTokenBondingV0";
        accounts: [{
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "UpdateTokenBondingV0Args";
            };
        }];
    } | {
        name: "buyV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyV0Args";
            };
        }];
    } | {
        name: "sellV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellV0Args";
            };
        }];
    }>;
    get wallet(): import("@project-serum/anchor/dist/cjs/provider").Wallet;
    get account(): anchor.AccountNamespace<SplTokenBondingIDL>;
    get errors(): Map<number, string>;
    sendInstructions(instructions: TransactionInstruction[], signers: Signer[], payer?: PublicKey): Promise<string>;
    initializeSolStorageInstructions(): Promise<InstructionResult<null>>;
    initializeSolStorage(): Promise<void>;
    initializeCurveInstructions({ payer, config: curveConfig, }: InitializeCurveArgs): Promise<InstructionResult<{
        curve: PublicKey;
    }>>;
    initializeCurve(args: InitializeCurveArgs): Promise<PublicKey>;
    tokenBondingKey(targetMint: PublicKey, index: number): Promise<[PublicKey, number]>;
    baseStorageAuthorityKey(tokenBonding: PublicKey): Promise<[PublicKey, number]>;
    createTokenBondingInstructions({ authority, payer, curve, baseMint, targetMint, baseStorage, buyBaseRoyalties, buyBaseRoyaltiesOwner, buyTargetRoyalties, buyTargetRoyaltiesOwner, sellBaseRoyalties, sellBaseRoyaltiesOwner, sellTargetRoyalties, sellTargetRoyaltiesOwner, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, mintCap, purchaseCap, goLiveDate, // 1 secs ago
    freezeBuyDate, targetMintDecimals, buyFrozen, index }: CreateTokenBondingArgs): Promise<InstructionResult<{
        tokenBonding: PublicKey;
        targetMint: PublicKey;
        buyBaseRoyalties: PublicKey;
        buyTargetRoyalties: PublicKey;
        sellBaseRoyalties: PublicKey;
        sellTargetRoyalties: PublicKey;
        baseStorage: PublicKey;
    }>>;
    accountExists(account: anchor.web3.PublicKey): Promise<boolean>;
    createTokenBonding(args: CreateTokenBondingArgs): Promise<PublicKey>;
    updateTokenBondingInstructions({ tokenBonding, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, buyBaseRoyalties, buyTargetRoyalties, sellBaseRoyalties, sellTargetRoyalties, authority, buyFrozen, }: UpdateTokenBondingArgs): Promise<InstructionResult<null>>;
    updateTokenBonding(args: UpdateTokenBondingArgs): Promise<void>;
    createTemporaryWSolAccount({ payer, owner, amount, }: {
        owner: PublicKey;
        payer: PublicKey;
        amount: number;
    }): Promise<{
        signer: Keypair;
        firstInstructions: TransactionInstruction[];
        lastInstructions: TransactionInstruction[];
    }>;
    buyInstructions({ tokenBonding, source, sourceAuthority, destination, desiredTargetAmount, baseAmount, slippage, payer }: BuyArgs): Promise<InstructionResult<null>>;
    buy(args: BuyArgs): Promise<void>;
    getState(): Promise<ProgramStateV0 | null>;
    sellInstructions({ tokenBonding, source, sourceAuthority, destination, targetAmount, slippage, payer }: SellArgs): Promise<InstructionResult<null>>;
    sell(args: SellArgs): Promise<void>;
    getPricing(tokenBonding: PublicKey): Promise<Curve>;
    getCurve(key: PublicKey, baseStorage: AccountInfo, baseMint: MintInfo, targetMint: MintInfo): Promise<Curve>;
}
//# sourceMappingURL=index.d.ts.map