"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.baseToTargetLogCurve = exports.targetToBaseLogCurve = exports.startFinishLogCurve = exports.amountAsNum = exports.asDecimal = exports.supplyAsNum = void 0;
const spl_token_1 = require("@solana/spl-token");
// @ts-ignore
const lambertw_1 = require("./lambertw");
function generalLogCurve(c, g, x) {
    return c * ((1 / g + x) * Math.log(1 + g * x) - x);
}
/// Integral of c * log(1 + g * x) dx from a to b
/// https://www.wolframalpha.com/input/?i=c+*+log%281+%2B+g+*+x%29+dx
function logCurveRange(c, g, a, b) {
    return generalLogCurve(c, g, b) - generalLogCurve(c, g, a);
}
function supplyAsNum(mint) {
    return amountAsNum(mint.supply, mint);
}
exports.supplyAsNum = supplyAsNum;
function asDecimal(percent) {
    return percent / 4294967295; // uint32 max value
}
exports.asDecimal = asDecimal;
function amountAsNum(amount, mint) {
    const decimals = new spl_token_1.u64(Math.pow(10, mint.decimals).toString());
    const decimal = amount.mod(decimals).toNumber() / decimals.toNumber();
    return amount.div(decimals).toNumber() + decimal;
}
exports.amountAsNum = amountAsNum;
const startFinishLogCurve = (curve) => (start, finish) => {
    const c = curve.c.toNumber() / 1000000000000;
    const g = curve.g.toNumber() / 1000000000000;
    return logCurveRange(c, g, start, finish);
};
exports.startFinishLogCurve = startFinishLogCurve;
const targetToBaseLogCurve = (curve, target, baseRoyaltiesPercent, targetRoyaltiesPercent) => (targetAmount) => {
    const targetRoyaltiesDecimal = asDecimal(targetRoyaltiesPercent);
    const price = exports.startFinishLogCurve(curve)(supplyAsNum(target), supplyAsNum(target) + targetAmount * (1 + targetRoyaltiesDecimal));
    return price * (1 + asDecimal(baseRoyaltiesPercent));
};
exports.targetToBaseLogCurve = targetToBaseLogCurve;
/*
Just accept the magic...
This might help if you can't
https://www.wolframalpha.com/input/?i=solve%5Bc*%281%2Fg+%2B+%28s+%2B+x%29%29+*+log%28g+*+%28s+%2B+x%29+%2B+1%29+-+c+*+%28s+%2B+x%29+%3D+k%2C+x%5D
*/
const baseToTargetLogCurve = (curve, target, baseRoyaltiesPercent, targetRoyaltiesPercent) => (baseAmount) => {
    const royaltySubtractedBaseAmount = baseAmount * (1 - asDecimal(baseRoyaltiesPercent));
    const c = curve.c.toNumber() / 1000000000000;
    const g = curve.g.toNumber() / 1000000000000;
    const s = supplyAsNum(target);
    const rewardsDecimal = asDecimal(targetRoyaltiesPercent);
    const k = royaltySubtractedBaseAmount + generalLogCurve(c, g, s);
    const exp = lambertw_1.gsl_sf_lambert_W0((g * k - c) / (c * Math.E)) + 1;
    return Math.abs((Math.pow(Math.E, exp) - g * s - 1) / ((1 + rewardsDecimal) * g));
};
exports.baseToTargetLogCurve = baseToTargetLogCurve;
//# sourceMappingURL=pricing.js.map